
subroutine CalcEnergy(Pos, M, L, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
	!dimensionless
    real(8), parameter :: k = 3000., r0 = 2.46/2.43, B = 23.3/20.4, delta = 1.09/2.43
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d2, id2, id6, id12
	real(8) :: d
	real(8) :: rc2, Shift
	integer :: i, j
	PEnergy = 0.
	
	Shift = -4. * (rc**(-12) - rc**(-6))
	rc2 = rc * rc
	
	do i = 0, NAtom - 1
		!store Pos(i,:) in a temporary array for faster access in j loop
		Posi = Pos(i,:)
			do j = i + 1, NAtom - 1
				rij = Pos(j,:) - Posi
				rij = rij - L * dnint(rij/L)
								
				!this if condition will never hold true for this project
				!check whether atoms are bonded or not
				!if (j == i + 1 .and. mod(j,M) > 0) then
				!	!harmonic
					d = sqrt(sum(rij*rij))
				!	PEnergy = PEnergy + (k/2)*(d - r0)**2
				!else
					!lennard-jones + gaussian
					d = sqrt(sum(rij*rij))
					d2 = sum(rij * rij)
					if (d2 < rc2) then
						id2 = 1. / d2
						id6 = id2 * id2 * id2
						id12 = id6 * id6
						PEnergy = PEnergy + 4. * (id12 - id6) + Shift + B * exp((-(d - r0)**2)/delta**2)
					endif
				!end if
				
			enddo
	
	enddo
end subroutine

subroutine CalcEnergyOnePosition(Pos, M, L, rc, PEnergy, Dim, NAtom, displacedAtomNumber)
    implicit none
    integer, intent(in) :: M, Dim, NAtom, displacedAtomNumber
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 2.46/2.43, B = 23.3/20.4, delta = 1.09/2.43
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d2, id2, id6, id12
	real(8) :: d
	real(8) :: rc2, Shift
	integer :: i, j
	PEnergy = 0.
	
	Shift = -4. * (rc**(-12) - rc**(-6))
	rc2 = rc * rc
	
	do i = displacedAtomNumber, displacedAtomNumber
		!store Pos(i,:) in a temporary array for faster access in j loop
		Posi = Pos(i,:)
			do j = 0, NAtom - 1
				if ( j /= displacedAtomNumber  ) then
					rij = Pos(j,:) - Posi
					rij = rij - L * dnint(rij/L)
							
					
					!check whether atoms are bonded or not
					!if (j == i + 1 .and. mod(j,M) > 0)  then
					!	!harmonic
						d = sqrt(sum(rij*rij))
					!	PEnergy = PEnergy + (k/2)*(d - r0)**2
					!else if (j == i - 1 .and. mod(i,M) > 0) then
					!	!harmonic
					!	d = sqrt(sum(rij*rij))
					!	PEnergy = PEnergy + (k/2)*(d - r0)**2
					!else
						!lennard-jones
						d2 = sum(rij * rij)
						if (d2 < rc2) then
							id2 = 1. / d2
							id6 = id2 * id2 * id2
							id12 = id6 * id6
							PEnergy = PEnergy + 4. * (id12 - id6) + Shift + B * exp((-(d - r0)**2)/delta**2)
						endif
					!end if
				end if		
			enddo
	
	enddo
end subroutine

subroutine CalcEnergyForces(Pos, M, L, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), parameter :: k = 3000., r0 = 2.46/2.43, B = 23.3/20.4, delta = 1.09/2.43
	real(8), dimension(Dim) :: rij, Fij, Posi
    real(8) :: d2, id2, id6, id12
	real(8) :: d
	real(8) :: rc2, Shift
	integer :: i, j
	PEnergy = 0.
	Forces = 0.
	
	Shift = -4. * (rc**(-12) - rc**(-6))
	rc2 = rc * rc
	
	do i = 0, NAtom - 1
		!store Pos(i,:) in a temporary array for faster access in j loop
		Posi = Pos(i,:)
			do j = i + 1, NAtom - 1
				rij = Pos(j,:) - Posi
				rij = rij - L * dnint(rij/L)
								
				d = sqrt(sum(rij*rij))
				
				!check whether atoms are bonded or not
				!if (j == i + 1 .and. mod(j,M) > 0) then
				!	!harmonic
				
				!	PEnergy = PEnergy + (k/2)*(d - r0)**2
				!	Fij = rij * k * ((d - r0) / d)
				!	Forces(i,:) = Forces(i,:) + Fij
				!	Forces(j,:) = Forces(j,:) - Fij
				!else
					!lennard-jones
					d2 = sum(rij * rij)
					if (d2 < rc2) then
						id2 = 1. / d2
						id6 = id2 * id2 * id2
						id12 = id6 * id6
						PEnergy = PEnergy + 4. * (id12 - id6) + Shift + B * exp((-(d - r0)**2)/delta**2)
						Fij = rij * ((-48. * id12 + 24. * id6) * id2)
						Forces(i,:) = Forces(i,:) + Fij
						Forces(j,:) = Forces(j,:) - Fij
					endif
				!end if
				
				
			enddo
	enddo
	
end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, M, L, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, M, L, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine

subroutine CalcEnergySolvation(Pos, M, L, rc, PEnergy, Dim, NAtom, displacedAtomNumber)
    implicit none
    integer, intent(in) :: M, Dim, NAtom, displacedAtomNumber
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., eps1 = (2*.89)/20.4, sigma1 = 3.45/2.43
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d2, id2, id6, id12, qd
	real(8) :: rc2, Shift
	integer :: i, j
	PEnergy = 0.
	
	qd = sigma1/rc
	Shift = -4. * eps1 * (qd**(12) - qd**(6))
	rc2 = rc * rc
	
	do i = displacedAtomNumber, displacedAtomNumber
		!store Pos(i,:) in a temporary array for faster access in j loop
		Posi = Pos(i,:)
			do j = 0, NAtom - 1
				if ( j /= displacedAtomNumber  ) then
					rij = Pos(j,:) - Posi
					rij = rij - L * dnint(rij/L)
							
					
					!check whether atoms are bonded or not
					!if (j == i + 1 .and. mod(j,M) > 0)  then
					!	!harmonic
					!	d = sqrt(sum(rij*rij))
					!	PEnergy = PEnergy + (k/2)*(d - r0)**2
					!else if (j == i - 1 .and. mod(i,M) > 0) then
					!	!harmonic
					!	d = sqrt(sum(rij*rij))
					!	PEnergy = PEnergy + (k/2)*(d - r0)**2
					!else
						!lennard-jones
						d2 = sum(rij * rij)
						if (d2 < rc2) then
							id2 = (sigma1*sigma1) / d2
							id6 = id2 * id2 * id2
							id12 = id6 * id6
							PEnergy = PEnergy + 4. * eps1 * (id12 - id6) + Shift
						endif
					!end if
				end if		
			enddo
	
	enddo
end subroutine